' ****************
' * Star Split   *
' * By Ted Felix *
' ****************

' Copyright 1980, 1982, 1992, 2003, Ted Felix, All Rights Reserved

' Star field simulation.  Based on the BASICA version dated 10/16/1982.
' Using the algorithm developed by Charles P. Felix.  Converted to QBASIC
' on 1/30/2003.

' The BASICA version only allowed for 11 stars and used the old CGA
' 320x200 graphics mode.  It barely gave the effect of a moving starfield
' on an original 4.77MHz IBM PC.  Now, on a 600MHz Pentium III, this
' program actually rocks.

' I originally wrote this for the TRS-80 Model I.  It was incredibly
' slow, but very cool.  Must've been before June 1982.  The C version
' claims some time in 1980.  Around November
' 1992, I wrote a version of this in C using integers.
' It was very fast and impressive.  It seems like I
' revisit this program every 10 years or so.  It will be interesting to
' see what the 2012 version will be like.

' Seed the random number generator
RANDOMIZE TIMER

' The handy dandy random function
DEF FNRAND (X) = INT(RND * X) + 1

CONST MaxStars = 500

' Star rates
DIM Rate(MaxStars)

' Star positions
DIM XS!(MaxStars), YS!(MaxStars)

' Previous star positions for cleanup
DIM PX%(MaxStars), PY%(MaxStars)

' Rates less than 1 will make the stars go in.
CONST RateMin = 1
CONST RateRange = .05

' Set up the proper graphics screen
SCREEN 12
' Screen size
CONST XSize = 640
CONST YSize = 480
' Number of colors
CONST Colors = 16
' Center of the screen
CONST XCenter = XSize / 2
CONST YCenter = YSize / 2
' Area in the center where new stars appear
CONST XCenterRange = XSize / 10
CONST YCenterRange = YSize / 10

CLS
KEY OFF

' Ask how many stars the user wants to see
DO
  PRINT "Number of stars ( 1 -"; MaxStars; "):";
  INPUT " ", N
LOOP UNTIL N <= MaxStars AND N >= 1

CLS

' Set up the initial star field

' For each star
FOR I = 0 TO N - 1
  ' Pick a rate
  Rate(I) = RND * RateRange + RateMin
  DO
    ' Pick a position
    XS!(I) = FNRAND(XSize)
    YS!(I) = FNRAND(YSize)

    ' If the position is the center, try again
  LOOP WHILE XS!(I) = XCenter AND YS!(I) = YCenter

  ' Initialize the previous position as well
  PX%(I) = INT(XS!(I))
  PY%(I) = INT(YS!(I))

NEXT I

' The main drawing loop

DO
  ' For each star
  FOR I = 0 TO N - 1
    ' Move the star away from the center
    XS!(I) = (XS!(I) - XCenter) * Rate(I) + XCenter
    YS!(I) = (YS!(I) - YCenter) * Rate(I) + YCenter

    ' If the star has gone beyond the edge of the screen
    IF XS!(I) > XSize OR YS!(I) > YSize OR XS!(I) < 0 OR YS!(I) < 0 THEN
      DO
        ' Pick a new location near the center
        XS!(I) = FNRAND(XCenterRange) + XCenter - XCenterRange / 2
        YS!(I) = FNRAND(YCenterRange) + YCenter - YCenterRange / 2

        ' Try again if the position picked was the center
      LOOP WHILE XS!(I) = XCenter AND YS!(I) = YCenter
    END IF

    ' Convert star position to integer
    X% = INT(XS!(I))
    Y% = INT(YS!(I))

    ' Clear the old
    PRESET (PX%(I), PY%(I))
    ' Draw the new
    PSET (X%, Y%), FNRAND(Colors)

    ' Save the current star position so we can clear the old later
    PX%(I) = X%
    PY%(I) = Y%
  NEXT I
LOOP WHILE INKEY$ = ""

